% Transformer Leakager Flux Study using FEMM - Praviraj PG (pravirajpg@gmail.com)
% Thanks to Dr. David Meeker, http://www.femm.info/wiki/DavidMeeker
% Please read README before using this code..
% Core-LV-HV...

job = input('Sheet Name # : ', 's');        % Excel Sheet Name..

% Reading Data from Excel..
data1 = xlsread('femmppg',job,'A3:H3');
data2 = xlsread('femmppg',job,'B6:H8');
data3 = xlsread('femmppg',job,'B11:H17');
data4 = xlsread('femmppg',job,'B20:Q23');

% Variables & Constants...
Cond = 50;                      % Conductivity of Cu... MS/m
muo = pi*4e-7;                  % Permeability of Air/Oil...
mur = 4e4;                      % Relative Permeability of Core...
J = 0;                          % Source Current Density, A/mm2..

% Set unit - inches or mm..
if data1(1,1) == 1
    unit = 'inches';
else
    unit = 'millimeters';
end

MVA = data1(1,2);                   % Transformer Base MVA..
f = data1(1,3);                     % Frequency..
Di = data1(1,4);                    % Core Diameter..
R = data1(1,5) + 3;                 % R - boundary or approx (Window Width + 5)..
Z = data1(1,6);                     % Z - boundary or Window Height..
nt = data1(1,7);                    % No. of Terminals..
nl = data1(1,8);                    % No. of Layers..

% Initialize... 
openfemm;                                   % Open the FEMM program...
main_resize(600,600);                       % Resize FEMM window..
newdocument(0);                             % Create a new document...
mi_probdef(0,unit,'axi',1e-8,1,30);         % Define Problem..

% Draw a rectangle to use as the Outer boundary and assign boundary condition..
mi_drawrectangle(0, 0, R, Z+2);                               % Boundary...
mi_addboundprop('Asymptotic', 0, 0, 0, 0, 0, 0, 1/(muo*R), 0, 2);
mi_selectsegment(R,Z);
mi_setsegmentprop('Asymptotic', 1, 1, 0, 0);

% Add medium, say Oil and Label it..
mi_addmaterial('Oil', 1, 1, 0, 0, 0, 0, 0, 1, 0, 0, 0);     % The medium
mi_addblocklabel(R-3,Z-2);                                  % Label it as Oil/Air..
mi_selectlabel(R-3,Z-2);
mi_setblockprop('Oil', 0, 1, '<None>', 0, 0, 0);
mi_clearselected
% Now, the finished input geometry can be displayed.
mi_zoomnatural


% Add Core, Draw and Label it..
Ri = Di/2;                           % Radius of Iron Circle..
mi_drawrectangle([1 1; Ri Z]);      % Draw rectangle for Core..
mi_addblocklabel(Ri/2,Z/2);             % Label..
mi_addmaterial('Iron', mur, mur, 0, 0, 0, 0.3556, 0, 0.965, 1, 0, 0);
mi_selectlabel(Ri/2,Z/2);
mi_setblockprop('Iron', 0, 1, '<None>', 0, 0, 0);
mi_clearselected

% Add Winding Properties..
mi_addmaterial('Coil', 1, 1, 0, J, Cond, 0, 0, 1, 0, 0, 0);

% Add each segments..
for ly = 1:nl
    tr = data3(1,ly);           % Terminal Number..
    ir = data3(2,ly)/2;         % Inside Radius..
    rd = data3(3,ly);           % Radial..
    fs = data3(4,ly);           % First Segment..
    ls = data3(5,ly);           % Last Segment..
    np = data3(6,ly);           % No. of Parallel Groups..
    cd = data3(7,ly);           % Current Direction..
    
    con = data2(1,tr);          % Connection Type..
    mva = data2(2,tr);          % 3-Phase MVA of Terminal..
    kV = data2(3,tr);           % Voltage of Terminal (Line-Line in kV)..
    if con == 1                 % If Wye Connected..
        Ip = mva*1000/sqrt(3)/kV;   % Coil Current..
    else                        % If Delta Connected..
        Ip = mva*1000/3/kV;   % Coil Current..
    end
    Ip = Ip/np;                 % Coil Current per Group..
    
    % Add each sections..
    for sn = fs:1:ls
        mi_drawrectangle([ir data4(1,sn); ir+rd data4(2,sn)]);         % Draw rectangle..
        mi_addblocklabel(ir+rd/2,(data4(2,sn)+data4(1,sn))/2);           % Label..
        str = ['seg' int2str(sn)];
        % Number of Active Turns..
        at = data4(4,sn);
        if at == 0              % If Active Turns are zero, make current zero/ some how FEMM takes it
            mi_addcircprop(str, 0, 1);      % as 1 Turn even if we put 0 turns..
        else
            mi_addcircprop(str, Ip, 1);
        end
        mi_selectlabel(ir+rd/2,(data4(2,sn)+data4(1,sn))/2);
        mi_setblockprop('Coil', 0, 1, str, 0, 1, at*cd);
        mi_clearselected
    end
end
        
% We have to give the geometry a name before we can analyze it.
mi_saveas('femmxfr.fem');

% Now,analyze the problem and load the solution when the analysis is finished
mi_analyze;
mi_loadsolution;

% Show flux density w/ vector plot..
mo_showdensityplot(1,0,0,0.1,'mag')
mo_showvectorplot(5,1)
mo_hidepoints()

% Leakage Reactance Calculation..
% Flux Linkage Method..
phi = 0;  ix = 0;
ns = max(data3(5,1:end));   % Total segments..
for sn = 1:ns
    str = ['seg' int2str(sn)];
    pval = mo_getcircuitproperties(str);
    phi = phi + pval(3)*pval(1);        % Leakage Flux x Current...
    ix = ix + pval(2)*pval(1);
end
X = phi *(2 * pi * f) / (MVA*1e6/3)*100;    % Leakage Reactance in %..
fprintf('Percentage Reactance (Flux Linkage) = %2.2f%% @ %2.3f MVA',X,MVA); fprintf('\n');

% Energy Method..
mo_groupselectblock;
E = mo_blockintegral(2);   % Energy (E=1/2LI2) => L = 2 x E / I^2..
mo_clearblock;
X = 2 * E *(2 * pi * f) / (MVA*1e6/3)*100;    % Leakage Reactance in %..
fprintf('Percentage Reactance (Energy Method) = %2.2f%% @ %2.3f MVA',X,MVA); fprintf('\n');